@@ -18,15 +18,17 @@ import mock
 
 from nova import exception
 from nova import test
+from oslo_config import cfg
 from pypowervm import exceptions as pvm_ex
 from pypowervm.tests import test_fixtures as pvm_fx
-from pypowervm import util as pvm_util
 from pypowervm.wrappers import logical_partition as pvm_lpar
 from pypowervm.wrappers import managed_system as pvm_ms
 from pypowervm.wrappers import network as pvm_net
 
 from nova_powervm.virt.powervm import vif
 
+CONF = cfg.CONF
+
 
 def cna(mac):
     """Builds a mock Client Network Adapter for unit tests."""
@@ -45,8 +47,34 @@ class TestVifFunctions(test.TestCase):
             traits=pvm_fx.LocalPVMTraits)).adpt
         self.slot_mgr = mock.Mock()
 
+    @mock.patch('oslo_serialization.jsonutils.dumps')
+    @mock.patch('pypowervm.wrappers.event.Event')
+    def test_push_vif_event(self, mock_event, mock_dumps):
+        mock_vif = mock.Mock(mac='MAC', href='HREF')
+        vif._push_vif_event(self.adpt, 'action', mock_vif)
+        mock_dumps.assert_called_once_with(
+            {'provider': 'NOVA_PVM_VIF', 'action': 'action', 'mac': 'MAC'})
+        mock_event.bld.assert_called_once_with(self.adpt, 'HREF',
+                                               mock_dumps.return_value)
+        mock_event.bld.return_value.create.assert_called_once_with()
+
+        mock_dumps.reset_mock()
+        mock_event.bld.reset_mock()
+        mock_event.bld.return_value.create.reset_mock()
+
+        # Exception reraises
+        mock_event.bld.return_value.create.side_effect = IndexError
+        self.assertRaises(IndexError, vif._push_vif_event, self.adpt, 'action',
+                          mock_vif)
+        mock_dumps.assert_called_once_with(
+            {'provider': 'NOVA_PVM_VIF', 'action': 'action', 'mac': 'MAC'})
+        mock_event.bld.assert_called_once_with(self.adpt, 'HREF',
+                                               mock_dumps.return_value)
+        mock_event.bld.return_value.create.assert_called_once_with()
+
     @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
-    def test_plug(self, mock_bld_drv):
+    @mock.patch('nova_powervm.virt.powervm.vif._push_vif_event')
+    def test_plug(self, mock_event, mock_bld_drv):
         """Test the top-level plug method."""
         mock_vif = {'address': 'MAC'}
         slot_mgr = mock.Mock()
@@ -62,6 +90,7 @@ class TestVifFunctions(test.TestCase):
                                                                new_vif=True)
         slot_mgr.register_vnet.assert_called_once_with(
             mock_bld_drv.return_value.plug.return_value)
+        mock_event.assert_called_once_with(self.adpt, 'plug', vnet)
         self.assertEqual(mock_bld_drv.return_value.plug.return_value, vnet)
 
         # Clean up
@@ -69,9 +98,12 @@ class TestVifFunctions(test.TestCase):
         slot_mgr.build_map.get_vnet_slot.reset_mock()
         mock_bld_drv.return_value.plug.reset_mock()
         slot_mgr.register_vnet.reset_mock()
+        mock_event.reset_mock()
 
-        # 2) Without slot registration
+        # 2) Without slot registration; and plug returns None (which it should
+        # IRL whenever new_vif=False).
         slot_mgr.build_map.get_vnet_slot.return_value = 123
+        mock_bld_drv.return_value.plug.return_value = None
         vnet = vif.plug(self.adpt, 'host_uuid', 'instance', mock_vif, slot_mgr,
                         new_vif=False)
 
@@ -81,36 +113,43 @@ class TestVifFunctions(test.TestCase):
         mock_bld_drv.return_value.plug.assert_called_once_with(mock_vif, 123,
                                                                new_vif=False)
         slot_mgr.register_vnet.assert_not_called()
-        self.assertEqual(mock_bld_drv.return_value.plug.return_value, vnet)
+        mock_event.assert_not_called()
+        self.assertIsNone(vnet)
 
     @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
-    def test_unplug(self, mock_bld_drv):
+    @mock.patch('nova_powervm.virt.powervm.vif._push_vif_event')
+    def test_unplug(self, mock_event, mock_bld_drv):
         """Test the top-level unplug method."""
+        mock_vif = {'address': 'MAC'}
         slot_mgr = mock.Mock()
 
         # 1) With slot deregistration, default cna_w_list
         mock_bld_drv.return_value.unplug.return_value = 'vnet_w'
-        vif.unplug(self.adpt, 'host_uuid', 'instance', 'vif', slot_mgr)
+        vif.unplug(self.adpt, 'host_uuid', 'instance', mock_vif, slot_mgr)
         mock_bld_drv.assert_called_once_with(self.adpt, 'host_uuid',
-                                             'instance', 'vif')
+                                             'instance', mock_vif)
         mock_bld_drv.return_value.unplug.assert_called_once_with(
-            'vif', cna_w_list=None)
+            mock_vif, cna_w_list=None)
         slot_mgr.drop_vnet.assert_called_once_with('vnet_w')
+        mock_event.assert_called_once_with(self.adpt, 'unplug', 'vnet_w')
 
         # Clean up
         mock_bld_drv.reset_mock()
         mock_bld_drv.return_value.unplug.reset_mock()
         slot_mgr.drop_vnet.reset_mock()
+        mock_event.reset_mock()
 
         # 2) Without slot deregistration, specified cna_w_list
         mock_bld_drv.return_value.unplug.return_value = None
-        vif.unplug(self.adpt, 'host_uuid', 'instance', 'vif', slot_mgr,
+        vif.unplug(self.adpt, 'host_uuid', 'instance', mock_vif, slot_mgr,
                    cna_w_list='cnalist')
         mock_bld_drv.assert_called_once_with(self.adpt, 'host_uuid',
-                                             'instance', 'vif')
+                                             'instance', mock_vif)
         mock_bld_drv.return_value.unplug.assert_called_once_with(
-            'vif', cna_w_list='cnalist')
+            mock_vif, cna_w_list='cnalist')
         slot_mgr.drop_vnet.assert_not_called()
+        # When unplug doesn't find a vif, we don't push an event
+        mock_event.assert_not_called()
 
     @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
     def test_plug_raises(self, mock_vif_drv):
@@ -212,6 +251,51 @@ class TestVifFunctions(test.TestCase):
                           vif._build_vif_driver, self.adpt, 'host_uuid',
                           mock_inst, {'type': 'bad'})
 
+    @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
+    def test_pre_live_migrate_at_source(self, mock_build_vif_drv):
+        mock_drv = mock.MagicMock()
+        mock_build_vif_drv.return_value = mock_drv
+        mock_vif = mock.MagicMock()
+
+        vif.pre_live_migrate_at_source(self.adpt, 'host_uuid', mock.Mock(),
+                                       mock_vif)
+
+        mock_drv.pre_live_migrate_at_source.assert_called_once_with(mock_vif)
+
+    @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
+    def test_rollback_live_migration_at_destination(self, mock_build_vif_drv):
+        mock_build_vif_drv.return_value = mock_drv = mock.MagicMock()
+        mock_vif, mappings = mock.MagicMock(), {}
+
+        vif.rollback_live_migration_at_destination(
+            self.adpt, 'host_uuid', mock.Mock(), mock_vif,
+            mappings)
+
+        rb = mock_drv.rollback_live_migration_at_destination
+        rb.assert_called_once_with(mock_vif, mappings)
+
+    @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
+    def test_pre_live_migrate_at_destination(self, mock_build_vif_drv):
+        mock_drv = mock.MagicMock()
+        mock_build_vif_drv.return_value = mock_drv
+        mock_vif = mock.MagicMock()
+
+        vif.pre_live_migrate_at_destination(self.adpt, 'host_uuid',
+                                            mock.Mock(), mock_vif, {})
+
+        mock_drv.pre_live_migrate_at_destination.assert_called_once_with(
+            mock_vif, {})
+
+    @mock.patch('nova_powervm.virt.powervm.vif._build_vif_driver')
+    def test_post_live_migrate_at_source(self, mock_build_vif_drv):
+        mock_drv = mock.MagicMock()
+        mock_build_vif_drv.return_value = mock_drv
+        mock_vif = mock.MagicMock()
+
+        vif.post_live_migrate_at_source(self.adpt, 'host_uuid', mock.Mock(),
+                                        mock_vif)
+        mock_drv.post_live_migrate_at_source.assert_called_once_with(mock_vif)
+
 
 class TestVifSriovDriver(test.TestCase):
 
@@ -243,8 +327,8 @@ class TestVifSriovDriver(test.TestCase):
             mac_addr=mock_san_mac.return_value, allowed_vlans='NONE',
             allowed_macs='NONE')
         mock_back_devs.assert_called_once_with(
-            mock_vnic_bld.return_value, ['port1', 'port2'], min_redundancy=3,
-            max_redundancy=3, capacity=None, check_port_status=True)
+            mock_vnic_bld.return_value, ['port1', 'port2'], redundancy=3,
+            capacity=None, check_port_status=True)
         mock_pvm_uuid.assert_called_once_with(self.drv.instance)
         mock_vnic_bld.return_value.create.assert_called_once_with(
             parent_type=pvm_lpar.LPAR, parent_uuid=mock_pvm_uuid.return_value)
@@ -262,8 +346,8 @@ class TestVifSriovDriver(test.TestCase):
             mac_addr=mock_san_mac.return_value, allowed_vlans='NONE',
             allowed_macs='NONE')
         mock_back_devs.assert_called_once_with(
-            mock_vnic_bld.return_value, ['port1', 'port2'], min_redundancy=3,
-            max_redundancy=3, capacity=0.08, check_port_status=True)
+            mock_vnic_bld.return_value, ['port1', 'port2'],
+            redundancy=3, capacity=0.08, check_port_status=True)
         mock_pvm_uuid.assert_called_once_with(self.drv.instance)
         mock_vnic_bld.return_value.create.assert_called_once_with(
             parent_type=pvm_lpar.LPAR, parent_uuid=mock_pvm_uuid.return_value)
@@ -427,14 +511,6 @@ class TestVifSeaDriver(test.TestCase):
         self.assertEqual(0, cnas[1].delete.call_count)
         self.assertEqual(1, cnas[2].delete.call_count)
 
-    def test_post_live_migrate_at_destination(self):
-        # Make sure the no-op works properly
-        self.drv.post_live_migrate_at_destination(mock.Mock())
-
-    def test_post_live_migrate_at_source(self):
-        # Make sure the no-op works properly
-        self.drv.post_live_migrate_at_source(mock.Mock())
-
 
 class TestVifLBDriver(test.TestCase):
 
@@ -654,72 +730,110 @@ class TestVifOvsDriver(test.TestCase):
         # Validate the OVS port delete call was made
         mock_del_ovs_port.assert_called_with('br-int', 'fake_dev')
 
+    @mock.patch('pypowervm.tasks.cna.find_trunks')
+    @mock.patch('pypowervm.wrappers.network.CNA.search')
+    @mock.patch('pypowervm.util.sanitize_mac_for_api')
+    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    def test_pre_live_migrate_at_source(self, mock_pvm_uuid, mock_sanitize,
+                                        mock_cna_search, mock_trunk_find):
+        # Set up the mocks
+        vif = {'address': 'aa:bb:cc:dd:ee:ff'}
+        mock_sanitize.return_value = 'AABBCCDDEEFF'
+        mock_trunk_find.return_value = 'trunk'
+        mock_pvm_uuid.return_value = 'pvm_uuid'
+
+        resp = self.drv.pre_live_migrate_at_source(vif)
+        self.assertEqual(resp, 'trunk')
+
+        # Make sure the APIs were called correctly
+        mock_sanitize.assert_called_once_with(vif['address'])
+        mock_cna_search.assert_called_once_with(
+            self.adpt, parent_type=pvm_lpar.LPAR.schema_type,
+            parent_uuid='pvm_uuid', one_result=True, mac='AABBCCDDEEFF')
+        mock_trunk_find.assert_called_once_with(self.adpt, mock.ANY)
+
     @mock.patch('nova.network.linux_net.create_ovs_vif_port')
     @mock.patch('nova.utils.execute')
-    @mock.patch('nova_powervm.virt.powervm.vif.PvmOvsVifDriver.'
-                'get_trunk_dev_name')
-    @mock.patch('nova_powervm.virt.powervm.vm.get_pvm_uuid')
+    @mock.patch('pypowervm.tasks.cna.crt_trunk_with_free_vlan')
     @mock.patch('pypowervm.tasks.partition.get_this_partition')
-    @mock.patch('pypowervm.wrappers.network.CNA.bld')
-    @mock.patch('pypowervm.wrappers.network.CNA.search')
-    @mock.patch('pypowervm.tasks.cna.assign_free_vlan')
-    @mock.patch('pypowervm.wrappers.network.VSwitch.search')
-    def test_post_live_migrate_at_destination(
-        self, mock_vswitch_search, mock_assign_vlan, mock_cna_search,
-        mock_trunk_bld, mock_mgmt_lpar, mock_uuid, mock_dev_name, mock_exec,
-        mock_crt_ovs_vif_port):
-        # Mock the vif argument
-        def getitem(name):
-            return fake_vif[name]
+    def test_pre_live_migrate_at_destination(
+            self, mock_part_get, mock_trunk_crt, mock_execute,
+            mock_crt_ovs_port):
+        # Mock the vif
+        vif = {'devname': 'tap-dev', 'address': 'aa:bb:cc:dd:ee:ff',
+               'network': {'bridge': 'br-int'}, 'id': 'vif_id'}
 
-        fake_vif = {'address': 'aa:bb:cc:dd:ee:ff',
-                    'network': {'bridge': 'br-int'}}
-        mock_vif = mock.MagicMock()
-        mock_vif.__getitem__.side_effect = getitem
+        # Mock out the management partition
+        mock_mgmt_wrap = mock.MagicMock()
+        mock_mgmt_wrap.uuid = 'mgmt_uuid'
+        mock_part_get.return_value = mock_mgmt_wrap
 
-        # Mock other values
-        mock_assign_vlan.return_value = mock.MagicMock(pvid=70, enabled=True)
-        mock_cna = mock.MagicMock(pvid=35, enabled=False)
-        mock_cna_search.return_value = mock_cna
-        mock_trunk = mock.MagicMock()
-        mock_trunk_bld.return_value = mock_trunk
-        mock_mgmt_lpar.return_value = mock.Mock(uuid='mgmt_uuid')
-        mock_vswitch = mock.MagicMock()
-        mock_vswitch_search.return_value = mock_vswitch
-        mock_uuid.return_value = 'lpar_uuid'
-        mock_dev_name.return_value = 'dev_name'
-        mac = pvm_util.sanitize_mac_for_api(fake_vif['address'])
-        self.drv.adapter = self.adpt
+        mock_trunk_crt.return_value = [mock.Mock(pvid=2)]
 
-        # Execute and verify the results
-        self.drv.post_live_migrate_at_destination(mock_vif)
-        mock_assign_vlan.assert_called_once_with(
-            self.drv.adapter, self.drv.host_uuid, mock_vswitch, mock_cna)
-        mock_trunk_bld(
-            self.drv.adapter, pvid=70, vlan_ids=[], vswitch=mock_vswitch)
-        mock_trunk.create.assert_called_once_with(
-            parent=mock_mgmt_lpar.return_value)
-        mock_cna_search.assert_called_once_with(
-            self.adpt, mac=mac, one_result=True,
-            parent_type=pvm_lpar.LPAR, parent_uuid='lpar_uuid')
+        # Invoke and test the basic response
+        vea_vlan_mappings = {}
+        self.drv.pre_live_migrate_at_destination(vif, vea_vlan_mappings)
+        self.assertEqual(vea_vlan_mappings['aa:bb:cc:dd:ee:ff'], 2)
+
+        # Now validate it called the things it needed to
+        mock_execute.assert_called_once_with('ip', 'link', 'set', 'tap-dev',
+                                             'up', run_as_root=True)
+        mock_trunk_crt.assert_called_once_with(
+            self.adpt, 'host_uuid', ['mgmt_uuid'],
+            CONF.powervm.pvm_vswitch_for_novalink_io, dev_name='tap-dev')
+        mock_crt_ovs_port.assert_called_once_with(
+            'br-int', 'tap-dev', 'vif_id', 'aa:bb:cc:dd:ee:ff', self.inst.uuid)
 
-    @mock.patch('pypowervm.tasks.cna.find_orphaned_trunks')
     @mock.patch('nova.network.linux_net.delete_ovs_vif_port')
-    @mock.patch('nova_powervm.virt.powervm.vif.PvmOvsVifDriver.'
-                'get_trunk_dev_name')
-    def test_post_live_migrate_at_source(self, mock_trunk_dev_name,
-                                         mock_del_ovs_port,
-                                         mock_find_orphan):
-        t1, t2 = mock.MagicMock(), mock.MagicMock()
-        mock_find_orphan.return_value = [t1, t2]
-        mock_trunk_dev_name.return_value = 'fake_dev1'
+    @mock.patch('pypowervm.wrappers.network.CNA.get')
+    @mock.patch('pypowervm.tasks.partition.get_this_partition')
+    @mock.patch('pypowervm.wrappers.network.VSwitch.search')
+    def test_rollback_live_migration_at_destination(
+            self, mock_vs_search, mock_get_part, mock_cna_get,
+            mock_delete_ovs_port):
+        # All the fun mocking
+        mock_vs_search.return_value = mock.MagicMock(switch_id=5)
 
-        mock_vif = {'network': {'bridge': 'br-int'}}
-        self.drv.post_live_migrate_at_source(mock_vif)
+        # Since this gets passed through conductor, the VLAN's switch to string
+        # format.
+        vea_vlan_mappings = {'aa:bb:cc:dd:ee:ff': '3',
+                             'aa:bb:cc:dd:ee:ee': '4'}
+        vif = {'devname': 'tap-dev', 'address': 'aa:bb:cc:dd:ee:ee',
+               'network': {'bridge': 'br-int'}, 'id': 'vif_id'}
 
-        # The orphans should have been deleted
-        self.assertTrue(t1.delete.called)
-        self.assertTrue(t2.delete.called)
+        mock_vio = mock.MagicMock(schema_type='VIO', uuid='uuid')
+        mock_get_part.return_value = mock_vio
 
-        # Validate the OVS port delete call was made twice
-        mock_del_ovs_port.assert_called_once_with('br-int', 'fake_dev1')
+        trunk1 = mock.Mock(pvid=2, vswitch_id=3, trunk_pri=1)
+        trunk2 = mock.Mock(pvid=3, vswitch_id=5, trunk_pri=1)
+        trunk3 = mock.Mock(pvid=4, vswitch_id=5, trunk_pri=None)
+        trunk4 = mock.Mock(pvid=4, vswitch_id=5, trunk_pri=1)
+        mock_cna_get.return_value = [trunk1, trunk2, trunk3, trunk4]
+
+        # Invoke
+        self.drv.rollback_live_migration_at_destination(vif, vea_vlan_mappings)
+
+        # Make sure the OVS port was deleted
+        mock_delete_ovs_port.assert_called_once_with('br-int', 'tap-dev')
+
+        # Make sure the trunk was deleted
+        trunk4.delete.assert_called_once()
+
+        # Now make sure the calls were done correctly to actually produce a
+        # trunk adapter
+        mock_vs_search.assert_called_once_with(
+            self.drv.adapter, parent_type=pvm_ms.System, one_result=True,
+            name=CONF.powervm.pvm_vswitch_for_novalink_io)
+        mock_get_part.assert_called_once_with(self.drv.adapter)
+        mock_cna_get.assert_called_once_with(
+            self.drv.adapter, parent=mock_vio)
+
+    @mock.patch('nova.network.linux_net.delete_ovs_vif_port')
+    def test_post_live_migrate_at_source(self, mock_delete_ovs_port):
+        # Mock the vif
+        vif = {'devname': 'tap-dev', 'address': 'aa:bb:cc:dd:ee:ff',
+               'network': {'bridge': 'br-int'}, 'id': 'vif_id'}
+        # Invoke and test
+        self.drv.post_live_migrate_at_source(vif)
+        # Now validate it called the things it needed to
+        mock_delete_ovs_port.assert_called_once_with('br-int', 'tap-dev')
