@@ -25,6 +25,7 @@ from nova_powervm.virt.powervm.i18n import _LE
 from nova_powervm.virt.powervm.i18n import _LI
 from nova_powervm.virt.powervm import vm
 
+from nova import image
 import pypowervm.const as pvm_const
 from pypowervm.tasks import cluster_ssp as tsk_cs
 from pypowervm.tasks import partition as tsk_par
@@ -37,6 +38,7 @@ import pypowervm.wrappers.storage as pvm_stg
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
+IMAGE_API = image.API()
 
 
 class SSPDiskAdapter(disk_drv.DiskAdapter):
@@ -227,11 +229,13 @@ class SSPDiskAdapter(disk_drv.DiskAdapter):
                  dict(image_type=image_type, image_id=image_meta.id,
                       instance_uuid=instance.uuid))
 
+        def upload(path):
+            IMAGE_API.download(context, image_meta.id, dest_path=path)
+
         image_lu = tsk_cs.get_or_upload_image_lu(
             self._tier, self._get_image_name(image_meta),
-            self._any_vios_uuid(),
-            lambda: self._get_image_upload(context, image_meta),
-            image_meta.size)
+            self._any_vios_uuid(), upload,
+            image_meta.size, upload_type=tsk_stg.UploadType.FUNC)
 
         boot_lu_name = self._get_disk_name(image_type, instance)
         LOG.info(_LI('SSP: Disk name is %s'), boot_lu_name)
